package iwocs;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.Path;

import static iwocs.graph.GraphIO.*;
import static iwocs.graph.GraphUtils.*;

public class App {
  public static void main(String[] args) {
    System.setProperty("org.graphstream.ui.renderer", "org.graphstream.ui.j2dviewer.J2DGraphRenderer");
    Graph g = loadGraph("lh", "lh.dgs", true);

    g.addAttribute("ui.antialias");
    g.addAttribute("ui.quality");
    g.addAttribute("ui.stylesheet", loadFile("style.css", true));

    System.out.println("Graph loaded:");
    System.out.printf("|V|: %d\n|E|: %d\n<d>: %.3f\n<C>: %.3f\n",
      g.getNodeCount(),
      g.getEdgeCount(),
      Toolkit.averageDegree(g),
      Toolkit.averageClusteringCoefficient(g)
    );

    // Path p = getShortestPath(g, Toolkit.randomNode(g), Toolkit.randomNode(g));
    Path p = getDiameterPathWithDijkstra(g);

    System.out.println("\nDiameter path calculated:");
    System.out.printf("|V|: %d\n|E|: %d\nd: %.2f\n",
      p.getNodeCount(),
      p.getEdgeCount(),
      p.getPathWeight("length")
    );

    for(Edge e : p.getEachEdge())
      e.setAttribute("ui.class", "path");

    for(Node n : p.getEachNode())
      n.addAttribute("ui.class", "path");

    g.display(false);
  }
}