package iwocs.graph;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Scanner;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceDGS;

public class GraphIO {
  public static Graph loadGraph(String graphId, String fileName, boolean isResource) {
    Graph g = new SingleGraph(graphId);

    FileSource fs = new FileSourceDGS();

    fs.addSink(g);

    try {
      InputStream stream;
      if (isResource)
        stream = App.class.getClassLoader().getResourceAsStream(fileName);
      else
        stream = new FileInputStream(fileName);

      fs.readAll(stream);
    } catch (IOException e) {
      e.printStackTrace();
    }

    return g;
  }

  /**
   * Export degrees distribution of a graph to a given file.
   * 
   * @param g the graph from which we get the degree distribution.
   * @param destFile the path to the file in which the distribution will be written.
   * @param normalized if the values of the distribution will be normalized or not.
   */
  public static void exportDegreeDistribution(Graph g, String destFile, boolean normalized) {
    int[] degreesDistribution = Toolkit.degreeDistribution(g);
    double[] convertedDistribution = new double[degreesDistribution.length];
  
    for(int i=0; i < degreesDistribution.length; i++)
      convertedDistribution[i] = normalized ? 
        (double)degreesDistribution[i] / g.getNodeCount() :
        (double)degreesDistribution[i];

    try(BufferedWriter w = new BufferedWriter(new FileWriter(new File(destFile)))) {
      for(int i=0; i < convertedDistribution.length; i++)
        if(convertedDistribution[i] != 0)
          w.write(String.format("%d\t%f\n", i, convertedDistribution[i]));
    } catch(Exception e) {
      e.printStackTrace();
    }
  }

  public static String loadFile(String fileName, boolean isResource) {
    StringBuilder sb = new StringBuilder();

    Scanner sc;

    try {
      if (isResource)
        sc = new Scanner(App.class.getClassLoader().getResourceAsStream(fileName));
      else
        sc = new Scanner(new File(fileName));

      while (sc.hasNextLine())
        sb.append(sc.nextLine());
    } catch (IOException e) {
      e.printStackTrace();
    }

    return sb.toString();
  }
}