package iwocs.graph;

import org.graphstream.algorithm.APSP;
import org.graphstream.algorithm.APSP.APSPInfo;
import org.graphstream.algorithm.Dijkstra;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.Path;

public class GraphUtils {
  public static Path getDiameterPathWithDijkstra(Graph g) {
    Path diameter = new Path();

    Dijkstra dij = new Dijkstra(Dijkstra.Element.EDGE, "result", "length");
    dij.init(g);

    for (Node n : g.getEachNode()) {
      dij.setSource(n);
      dij.compute();

      System.out.println("Shortest paths calculated!");

      for (Node target : g.getEachNode()) {
        Path p = dij.getPath(target);

        if (p.getPathWeight("length") > diameter.getPathWeight("length"))
          diameter = p;
      }
    }

    return diameter;
  }

  public static Path getDiameterPathWithAPSP(Graph g) {
    Path diameter = new Path();

    APSP apsp = new APSP(g, "length", false);

    apsp.compute();

    System.out.println("Shortest paths calculated!");

    for (Node n : g.getEachNode()) {
      APSPInfo info = n.getAttribute(APSPInfo.ATTRIBUTE_NAME);

      for (Node target : g.getEachNode()) {
        Path p = info.getShortestPathTo(target.getId());

        if (p.getPathWeight("length") > diameter.getPathWeight("length"))
          diameter = p;
      }
    }

    return diameter;
  }

  private static Path getShortestPath(Graph g, Node source, Node dest) {
    Dijkstra dij = new Dijkstra(Dijkstra.Element.EDGE, "result", "length");

    dij.init(g);
    dij.setSource(source);
    dij.compute();

    return dij.getPath(dest);
  }
}